/****************************************************************************
** Meta object code from reading C++ file 'ktreewidgetsearchline.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kitemviews-6.26.0/src/ktreewidgetsearchline.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ktreewidgetsearchline.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21KTreeWidgetSearchLineE_t {};
} // unnamed namespace

template <> constexpr inline auto KTreeWidgetSearchLine::qt_create_metaobjectdata<qt_meta_tag_ZN21KTreeWidgetSearchLineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KTreeWidgetSearchLine",
        "hiddenChanged",
        "",
        "QTreeWidgetItem*",
        "searchUpdated",
        "searchString",
        "caseSensitivityChanged",
        "Qt::CaseSensitivity",
        "caseSensitivity",
        "keepParentsVisibleChanged",
        "keepParentsVisible",
        "addTreeWidget",
        "QTreeWidget*",
        "treeWidget",
        "removeTreeWidget",
        "updateSearch",
        "pattern",
        "setCaseSensitivity",
        "setKeepParentsVisible",
        "value",
        "setSearchColumns",
        "QList<int>",
        "columns",
        "setTreeWidget",
        "setTreeWidgets",
        "QList<QTreeWidget*>",
        "treeWidgets",
        "_k_rowsInserted",
        "QModelIndex",
        "_k_treeWidgetDeleted",
        "_k_slotColumnActivated",
        "QAction*",
        "_k_slotAllVisibleColumns",
        "_k_queueSearch",
        "_k_activateSearch",
        "caseSensitity"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'hiddenChanged'
        QtMocHelpers::SignalData<void(QTreeWidgetItem *, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Signal 'searchUpdated'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'caseSensitivityChanged'
        QtMocHelpers::SignalData<void(Qt::CaseSensitivity)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'keepParentsVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'addTreeWidget'
        QtMocHelpers::SlotData<void(QTreeWidget *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'removeTreeWidget'
        QtMocHelpers::SlotData<void(QTreeWidget *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'updateSearch'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'updateSearch'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'setCaseSensitivity'
        QtMocHelpers::SlotData<void(Qt::CaseSensitivity)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'setKeepParentsVisible'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Slot 'setSearchColumns'
        QtMocHelpers::SlotData<void(const QList<int> &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'setTreeWidget'
        QtMocHelpers::SlotData<void(QTreeWidget *)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'setTreeWidgets'
        QtMocHelpers::SlotData<void(const QList<QTreeWidget*> &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot '_k_rowsInserted'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int) const>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 2 }, { QMetaType::Int, 2 }, { QMetaType::Int, 2 },
        }}),
        // Slot '_k_treeWidgetDeleted'
        QtMocHelpers::SlotData<void(QObject *)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 2 },
        }}),
        // Slot '_k_slotColumnActivated'
        QtMocHelpers::SlotData<void(QAction *)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 2 },
        }}),
        // Slot '_k_slotAllVisibleColumns'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot '_k_queueSearch'
        QtMocHelpers::SlotData<void(const QString &)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot '_k_activateSearch'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'caseSensitity'
        QtMocHelpers::PropertyData<Qt::CaseSensitivity>(35, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 2),
        // property 'keepParentsVisible'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KTreeWidgetSearchLine, qt_meta_tag_ZN21KTreeWidgetSearchLineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KTreeWidgetSearchLine::staticMetaObject = { {
    QMetaObject::SuperData::link<QLineEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KTreeWidgetSearchLineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KTreeWidgetSearchLineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21KTreeWidgetSearchLineE_t>.metaTypes,
    nullptr
} };

void KTreeWidgetSearchLine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KTreeWidgetSearchLine *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->hiddenChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->searchUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->caseSensitivityChanged((*reinterpret_cast<std::add_pointer_t<Qt::CaseSensitivity>>(_a[1]))); break;
        case 3: _t->keepParentsVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->addTreeWidget((*reinterpret_cast<std::add_pointer_t<QTreeWidget*>>(_a[1]))); break;
        case 5: _t->removeTreeWidget((*reinterpret_cast<std::add_pointer_t<QTreeWidget*>>(_a[1]))); break;
        case 6: _t->updateSearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->updateSearch(); break;
        case 8: _t->setCaseSensitivity((*reinterpret_cast<std::add_pointer_t<Qt::CaseSensitivity>>(_a[1]))); break;
        case 9: _t->setKeepParentsVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->setSearchColumns((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 11: _t->setTreeWidget((*reinterpret_cast<std::add_pointer_t<QTreeWidget*>>(_a[1]))); break;
        case 12: _t->setTreeWidgets((*reinterpret_cast<std::add_pointer_t<QList<QTreeWidget*>>>(_a[1]))); break;
        case 13: _t->d->_k_rowsInserted((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 14: _t->d->_k_treeWidgetDeleted((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 15: _t->d->_k_slotColumnActivated((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 16: _t->d->_k_slotAllVisibleColumns(); break;
        case 17: _t->d->_k_queueSearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->d->_k_activateSearch(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KTreeWidgetSearchLine::*)(QTreeWidgetItem * , bool )>(_a, &KTreeWidgetSearchLine::hiddenChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KTreeWidgetSearchLine::*)(const QString & )>(_a, &KTreeWidgetSearchLine::searchUpdated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KTreeWidgetSearchLine::*)(Qt::CaseSensitivity )>(_a, &KTreeWidgetSearchLine::caseSensitivityChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KTreeWidgetSearchLine::*)(bool )>(_a, &KTreeWidgetSearchLine::keepParentsVisibleChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Qt::CaseSensitivity*>(_v) = _t->caseSensitivity(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->keepParentsVisible(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCaseSensitivity(*reinterpret_cast<Qt::CaseSensitivity*>(_v)); break;
        case 1: _t->setKeepParentsVisible(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KTreeWidgetSearchLine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KTreeWidgetSearchLine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KTreeWidgetSearchLineE_t>.strings))
        return static_cast<void*>(this);
    return QLineEdit::qt_metacast(_clname);
}

int KTreeWidgetSearchLine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KTreeWidgetSearchLine::hiddenChanged(QTreeWidgetItem * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KTreeWidgetSearchLine::searchUpdated(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KTreeWidgetSearchLine::caseSensitivityChanged(Qt::CaseSensitivity _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KTreeWidgetSearchLine::keepParentsVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
